/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package KDDCup05;

import KDDCup05.termstools.ResultOfAParticularTermNew;
import KDDCup05.termstools.ResultOfAParticularTermNew.RetrievalCutOff;
import com.google.gson.JsonArray;
import com.google.gson.JsonParser;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import lemur.dataStructures.CategoryScoreRank;
import lemurproject.indri.QueryEnvironment;
import lemurproject.indri.ScoredExtentResult;
import sqlite.dataSources.Category;

/**
 *
 * @author drosatosgr
 */
public class LoadLabeler {
    
    public static enum ScoringFormula{
	ApplyClassificationWithExponentOfScores, //0
	ApplyClassificationWithCombinedPosibilities, //1
        ApplyClassificationWithNormalizedCombinedPosibilities, //2
        ApplyClassificationWithMajorityVoteMulticlassificationKNN, //3
        ApplyClassificationWithWeightVoteMulticlassificationKNN //4
    }
    

    public Hashtable<String, ArrayList<String>> categoriesPerQuery;
    public Hashtable<String, ArrayList<String>> queriesPerCategory;

    public LoadLabeler() {
        categoriesPerQuery = new Hashtable<String, ArrayList<String>>();
        queriesPerCategory = new Hashtable<String, ArrayList<String>>();
    }

    public static ArrayList<String> getOnlyQueries(String filepath) {
        ArrayList<String> queries = new ArrayList<String>();
        BufferedReader br = null;
        try {
            br = new BufferedReader(new InputStreamReader(new FileInputStream(filepath), "UTF-8"));
            String inputLine;
            while ((inputLine = br.readLine()) != null) {
                String[] val = inputLine.split("\t");

                if (val.length > 0) {
                    if (!queries.contains(val[0])) {
                        queries.add(val[0]);
                    } else {
                        Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, "Error on: " + inputLine);
                        break;
                    }
                } else {
                    Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, "Error on: " + inputLine);
                    break;
                }
            }
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, ex);
        } catch (FileNotFoundException ex) {
            Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                br.close();
            } catch (IOException ex) {
                Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return queries;
    }
    
    public void load(String filepath, HashSet<String> unclassifiedQueries) {

        BufferedReader br = null;
        try {
            br = new BufferedReader(new InputStreamReader(new FileInputStream(filepath), "UTF-8"));
            String inputLine;
            while ((inputLine = br.readLine()) != null) {
                String[] val = inputLine.split("\t");

                if (val.length > 0) {
                    if(!unclassifiedQueries.contains(val[0])){
                    if (!categoriesPerQuery.containsKey(val[0])) {
                        ArrayList<String> a = new ArrayList<String>();
                        for (int i = 1; i < val.length; i++) {
                            a.add(val[i]);
                            addToUniqueCategories(val[0], val[i]);
                        }
                        categoriesPerQuery.put(val[0], a);
                    } else {
                        Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, "Error on: " + inputLine);
                        break;
                    }
                    }
                } else {
                    Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, "Error on: " + inputLine);
                    break;
                }
            }
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, ex);
        } catch (FileNotFoundException ex) {
            Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                br.close();
            } catch (IOException ex) {
                Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public HashSet<String> classifyQuerySet(String indexPath, int totalResults, int topNCateg, ArrayList<String> queries, Hashtable<Integer, Category> categs, ScoringFormula sF,boolean excludeUnclassifiedQueries) {
        QueryEnvironment qenv = new QueryEnvironment();
        try {
            HashSet<String> unclassifiedQueries = new HashSet<String>();
            qenv.addIndex(indexPath);
            int total = 0;
            if(totalResults <= 0){
                total = (int) qenv.documentCount(); //200;
            }else{
                total = totalResults;
            }
            for (String q : queries) {
                //System.out.println("- Expanding Query: \"" + q + "\"");
                //
                //ResultOfAParticularTermNew r = new ResultOfAParticularTermNew();
                //r.retrievEverythingFromIndexSingleAndKWordsWithWindow(Testing.indexStats, q.replaceAll("[^a-z\\sA-Z0-9]", " ").trim().toLowerCase(), 16, 1, 10, RetrievalCutOff.MINDFSEED);
                //String expand = r.getFirstResultsAsString(15);
                
                System.out.print("- Classifing: \"" + q /*+ expand*/ + "\"");
                ScoredExtentResult[] scResult = qenv.runQuery((q/*+expand*/).replaceAll("[^a-z\\sA-Z0-9]", " ").trim().toLowerCase(), total);
                String[] doccateg = qenv.documentMetadata(scResult, "docat");
                ArrayList<String> results = new ArrayList<String>();
                if(sF == ScoringFormula.ApplyClassificationWithExponentOfScores){       
                    results = applyClassificationWithExponentOfScores(scResult, doccateg, topNCateg, categs);
                }else if(sF == ScoringFormula.ApplyClassificationWithCombinedPosibilities){
                    results = applyClassificationWithCombinedPosibilities(scResult, doccateg, topNCateg, categs);
                }else if(sF == ScoringFormula.ApplyClassificationWithNormalizedCombinedPosibilities){
                    results = applyClassificationWithNormalizedCombinedPosibilities(scResult, doccateg, topNCateg, categs);
                }else if(sF == ScoringFormula.ApplyClassificationWithMajorityVoteMulticlassificationKNN){
                    results = applyClassificationWithMajorityVoteMulticlassificationKNN(scResult, doccateg, topNCateg, categs);
                }else if(sF == ScoringFormula.ApplyClassificationWithWeightVoteMulticlassificationKNN){
                    results = applyClassificationWithWeightVoteMulticlassificationKNN(scResult, doccateg, topNCateg, categs);
                }
                
                if(results.size() <= 0){
                    unclassifiedQueries.add(q);
                    System.out.println(" -> Unclassified!");
                    if(!excludeUnclassifiedQueries){
                        categoriesPerQuery.put(q, results);
                        for (String cat : results) {
                            addToUniqueCategories(q, cat);
                        }    
                    }
                }else{
                    System.out.println(" -> Classified!");
                    categoriesPerQuery.put(q, results);
                    for (String cat : results) {
                        System.out.print("("+cat+") ");
                        addToUniqueCategories(q, cat);
                    }  
                    System.out.println();
                }
            }
            return unclassifiedQueries;
        } catch (Exception ex) {
            Logger.getLogger(LoadLabeler.class.getName()).log(Level.SEVERE, null, ex);
            return new HashSet<String>();
        }
    }
    
    private ArrayList<String> applyClassificationWithMajorityVoteMulticlassificationKNN(ScoredExtentResult[] scResult, String[] doccateg, int topN, Hashtable<Integer, Category> categs) {
        ArrayList<String> cat = new ArrayList<String>();

        Hashtable<Integer, Double> countLeafs = new Hashtable<Integer, Double>();
        //Double sumPosibility = 0.0;
        for (int r = 0; r < scResult.length; r++) {
            JsonArray ja = (JsonArray) new JsonParser().parse(doccateg[r]);
            for (int j = 0; j < ja.size(); j++) {
                CategoryScoreRank csr = new CategoryScoreRank(ja.get(j).getAsJsonObject());
                if (!countLeafs.containsKey(csr._id)) {
                    countLeafs.put(csr._id, 1.0);
                } else {
                    countLeafs.replace(csr._id, countLeafs.get(csr._id) + 1.0);
                }
            }
            //sumPosibility += Math.pow(Math.E, scResult[r].score);
            //if(sumPosibility >= 0.10) break;
        }

        Comparator comp = new Comparator<Map.Entry<Integer, Double>>() {
            public int compare(Map.Entry<Integer, Double> o1, Map.Entry<Integer, Double> o2) {
                return o1.getValue().compareTo(o2.getValue());
            }
        };

        ArrayList<Map.Entry<Integer, Double>> leafs = new ArrayList(countLeafs.entrySet());
        Collections.sort(leafs, comp);

        for (int i = 0; i < leafs.size() && cat.size() < topN; i++) {
            if(leafs.get(i).getValue() > (Double)(scResult.length / 6.0)) cat.add(categs.get(categs.get(leafs.get(i).getKey().intValue()).parent_id).name + "\\" + categs.get(leafs.get(i).getKey().intValue()).name);
        }

        return cat;
    }
    
    private ArrayList<String> applyClassificationWithWeightVoteMulticlassificationKNN(ScoredExtentResult[] scResult, String[] doccateg, int topN, Hashtable<Integer, Category> categs) {
        ArrayList<String> cat = new ArrayList<String>();

        Hashtable<Integer, Double> countLeafs = new Hashtable<Integer, Double>();
        Double sumPosibility = 0.0;
        for (int r = 0; r < scResult.length; r++) {
            JsonArray ja = (JsonArray) new JsonParser().parse(doccateg[r]);
            for (int j = 0; j < ja.size(); j++) {
                CategoryScoreRank csr = new CategoryScoreRank(ja.get(j).getAsJsonObject());
                if (!countLeafs.containsKey(csr._id)) {
                    countLeafs.put(csr._id, Math.pow(Math.E, scResult[r].score));
                } else {
                    countLeafs.replace(csr._id, countLeafs.get(csr._id) + Math.pow(Math.E, scResult[r].score));
                }
            }
            sumPosibility += Math.pow(Math.E, scResult[r].score);
            //if(sumPosibility >= 0.10) break;
        }

        Comparator comp = new Comparator<Map.Entry<Integer, Double>>() {
            public int compare(Map.Entry<Integer, Double> o1, Map.Entry<Integer, Double> o2) {
                return o1.getValue().compareTo(o2.getValue());
            }
        };

        ArrayList<Map.Entry<Integer, Double>> leafs = new ArrayList(countLeafs.entrySet());
        Collections.sort(leafs, comp);

        for (int i = 0; i < leafs.size() && cat.size() < topN; i++) {
            if(leafs.get(leafs.size() - (i + 1)).getValue() > (sumPosibility / 6.0)) cat.add(categs.get(categs.get(leafs.get(leafs.size() - (i + 1)).getKey().intValue()).parent_id).name + "\\" + categs.get(leafs.get(leafs.size() - (i + 1)).getKey().intValue()).name);
        }

        return cat;
    }

    private ArrayList<String> applyClassificationWithNormalizedCombinedPosibilities(ScoredExtentResult[] scResult, String[] doccateg, int topN, Hashtable<Integer, Category> categs) {
        ArrayList<String> cat = new ArrayList<String>();

        Hashtable<Integer, Double> countLeafs = new Hashtable<Integer, Double>();
        //Double sumPosibility = 0.0;
        for (int r = 0; r < scResult.length; r++) {
            JsonArray ja = (JsonArray) new JsonParser().parse(doccateg[r]);
            for (int j = 0; j < ja.size(); j++) {
                CategoryScoreRank csr = new CategoryScoreRank(ja.get(j).getAsJsonObject());
                if (!countLeafs.containsKey(csr._id)) {
                    countLeafs.put(csr._id, Math.pow(Math.E, scResult[r].score)*(Math.pow(Math.E, csr.score)/Math.pow(Math.E, csr.scoreOfFirstInRank)));
                } else {
                    countLeafs.replace(csr._id, countLeafs.get(csr._id) + Math.pow(Math.E, scResult[r].score)*(Math.pow(Math.E, csr.score)/Math.pow(Math.E, csr.scoreOfFirstInRank))); //*Math.pow(Math.E, csr.score/csr.scoreOfFirstInRank));
                }
            }
            //sumPosibility += Math.pow(Math.E, scResult[r].score);
            //if(sumPosibility >= 0.20) break;
        }

        Comparator comp = new Comparator<Map.Entry<Integer, Double>>() {
            public int compare(Map.Entry<Integer, Double> o1, Map.Entry<Integer, Double> o2) {
                return o1.getValue().compareTo(o2.getValue());
            }
        };

        ArrayList<Map.Entry<Integer, Double>> leafs = new ArrayList(countLeafs.entrySet());
        Collections.sort(leafs, comp);

        for (int i = 0; i < leafs.size() && i < topN; i++) {
            cat.add(categs.get(categs.get(leafs.get(leafs.size() - (i + 1)).getKey().intValue()).parent_id).name + "\\" + categs.get(leafs.get(leafs.size() - (i + 1)).getKey().intValue()).name);
        }

        return cat;
    }       
    
    private ArrayList<String> applyClassificationWithCombinedPosibilities(ScoredExtentResult[] scResult, String[] doccateg, int topN, Hashtable<Integer, Category> categs) {
        ArrayList<String> cat = new ArrayList<String>();

        Hashtable<Integer, Double> countLeafs = new Hashtable<Integer, Double>();
        //Double sumScore = 0.0;
        for (int r = 0; r < scResult.length; r++) {
            JsonArray ja = (JsonArray) new JsonParser().parse(doccateg[r]);
            for (int j = 0; j < ja.size(); j++) {
                CategoryScoreRank csr = new CategoryScoreRank(ja.get(j).getAsJsonObject());
                if (!countLeafs.containsKey(csr._id)) {
                    countLeafs.put(csr._id, Math.pow(Math.E, scResult[r].score)*Math.pow(Math.E, csr.score));
                } else {
                    countLeafs.replace(csr._id, countLeafs.get(csr._id) + Math.pow(Math.E, scResult[r].score)*Math.pow(Math.E, csr.score));
                }
            }
            //sumScore += Math.pow(Math.E, scResult[r].score);
        }

        Comparator comp = new Comparator<Map.Entry<Integer, Double>>() {
            public int compare(Map.Entry<Integer, Double> o1, Map.Entry<Integer, Double> o2) {
                return o1.getValue().compareTo(o2.getValue());
            }
        };

        ArrayList<Map.Entry<Integer, Double>> leafs = new ArrayList(countLeafs.entrySet());
        Collections.sort(leafs, comp);

        for (int i = 0; i < leafs.size() && i < topN; i++) {
            cat.add(categs.get(categs.get(leafs.get(leafs.size() - (i + 1)).getKey().intValue()).parent_id).name + "\\" + categs.get(leafs.get(leafs.size() - (i + 1)).getKey().intValue()).name);
        }

        return cat;
    }

    private ArrayList<String> applyClassificationWithExponentOfScores(ScoredExtentResult[] scResult, String[] doccateg, int topN, Hashtable<Integer, Category> categs) {
        ArrayList<String> cat = new ArrayList<String>();

        Hashtable<Integer, Double> countLeafs = new Hashtable<Integer, Double>();
        //Double sumPosibility = 0.0;
        for (int r = 0; r < scResult.length; r++) {
            JsonArray ja = (JsonArray) new JsonParser().parse(doccateg[r]);
            for (int j = 0; j < ja.size(); j++) {
                CategoryScoreRank csr = new CategoryScoreRank(ja.get(j).getAsJsonObject());
                if (!countLeafs.containsKey(csr._id)) {
                    countLeafs.put(csr._id, Math.pow(Math.E, scResult[r].score));
                } else {
                    countLeafs.replace(csr._id, countLeafs.get(csr._id) + Math.pow(Math.E, scResult[r].score));
                }
            }
            //sumPosibility += Math.pow(Math.E, scResult[r].score);
            //if(sumPosibility >= 0.10) break;
        }

        Comparator comp = new Comparator<Map.Entry<Integer, Double>>() {
            public int compare(Map.Entry<Integer, Double> o1, Map.Entry<Integer, Double> o2) {
                return o1.getValue().compareTo(o2.getValue());
            }
        };

        ArrayList<Map.Entry<Integer, Double>> leafs = new ArrayList(countLeafs.entrySet());
        Collections.sort(leafs, comp);

        for (int i = 0; i < leafs.size() && i < topN; i++) {
            cat.add(categs.get(categs.get(leafs.get(leafs.size() - (i + 1)).getKey().intValue()).parent_id).name + "\\" + categs.get(leafs.get(leafs.size() - (i + 1)).getKey().intValue()).name);
        }

        return cat;
    }

    private void addToUniqueCategories(String query, String cat) {
        if (queriesPerCategory.containsKey(cat)) {
            queriesPerCategory.get(cat).add(query);
        } else {
            ArrayList<String> queries = new ArrayList<String>();
            queries.add(query);
            queriesPerCategory.put(cat, queries);
        }
    }

    public boolean validate(Hashtable<Integer, Category> categs) {
        Enumeration<Integer> keys = categs.keys();
        ArrayList<String> tmp = new ArrayList<String>();
        while (keys.hasMoreElements()) {
            int k = keys.nextElement();
            if (categs.get(k).parent_id != null) {
                tmp.add(categs.get(categs.get(k).parent_id).name + "\\" + categs.get(k).name);
            }
        }

        for (String c : this.queriesPerCategory.keySet()) {
            if (!tmp.contains(c)) {
                System.err.println("Error: \"" + c + "\" doesn't exist!");
                return false;
            }
        }

        return true;
    }

    public void printAll() {
        Enumeration<String> k = categoriesPerQuery.keys();
        int count = 0;
        while (k.hasMoreElements()) {
            count++;
            String query = k.nextElement();
            System.out.println("- (" + count + ") " + query);
            ArrayList<String> a = categoriesPerQuery.get(query);
            for (String cc : a) {
                System.out.println(cc);
            }
        }
    }

    public void printUniqueCategories() {
        System.out.println("Unique Categories (" + queriesPerCategory.size() + "): ");
        for (String c : this.queriesPerCategory.keySet()) {
            System.out.println(c);
        }
    }

    public ArrayList<String> getQueries() {
        ArrayList<String> q = new ArrayList<String>();
        Enumeration<String> k = categoriesPerQuery.keys();
        while (k.hasMoreElements()) {
            q.add(k.nextElement());
        }
        return q;
    }

    public int sumOfQueriesPerCategory() {
        int sum = 0;
        for (String cat : queriesPerCategory.keySet()) {
            sum += queriesPerCategory.get(cat).size();
        }
        return sum;
    }

    public int sumOfQueriesPerCorrectlyTaggedCategory(LoadLabeler ll) {
        int sum = 0;
        Hashtable<String, ArrayList<String>> tmp = new Hashtable<String, ArrayList<String>>();

        Enumeration<String> k = this.categoriesPerQuery.keys();
        while (k.hasMoreElements()) {
            String query = k.nextElement();
            ArrayList<String> a = findMatches(this.categoriesPerQuery.get(query), ll.categoriesPerQuery.get(query));
            for (String cat : a) {
                if (tmp.containsKey(cat)) {
                    tmp.get(cat).add(query);
                } else {
                    ArrayList<String> queries = new ArrayList<String>();
                    queries.add(query);
                    tmp.put(cat, queries);
                }
            }
        }
        
        for (String cat : tmp.keySet()) {
            sum += tmp.get(cat).size();
        }
        return sum;
    }

    private ArrayList<String> findMatches(ArrayList<String> l0, ArrayList<String> l1) {
        ArrayList<String> ret = new ArrayList<String>();
        for (String c : l0) {
            if (l1.contains(c)) {
                ret.add(c);
            }
        }
        return ret;
    }
}
