package KDDCup05.termstools;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.PriorityQueue;
import java.util.Random;
import java.util.Scanner;

public class MySortedHashMap {

	public HashSet<KTermPhrase> hashset;
	public PriorityQueue<TermInfoNew> sortedList;
	public int capacity;
	public double min = -1.0;

	public MySortedHashMap(int capacity) {
		this.hashset = new HashSet<KTermPhrase>(capacity+5);
		Comparator<TermInfoNew> comparator = new Comparator<TermInfoNew>(){
			public int compare(TermInfoNew x, TermInfoNew y)
			{
				if((new Double(x.score)).compareTo(y.score) == 0){
					if((new Integer(x.hints)).compareTo(y.hints) == 0){
						return (new Integer(x.hintsMinDF)).compareTo(y.hintsMinDF);
					}else{
						return (new Integer(x.hints)).compareTo(y.hints);
					}
				}else{
					return (new Double(x.score)).compareTo(y.score);
				}	
			}	
		};

		this.sortedList = new PriorityQueue<TermInfoNew>(capacity+5,comparator);
		this.capacity = capacity;
	}

	public boolean put(KTermPhrase k, TermInfoNew v){ // Prosoxh den kanw elegxo ama yparxei hdh
		boolean ret = false;

		if(v.score >= min){
			//if(!this.hashset.contains(k)){ // To ebgale gia ligoterh poluplokothta
			this.hashset.add(k);
			this.sortedList.add(v);

			if(this.sortedList.size() > capacity){
				TermInfoNew ti = this.sortedList.poll();
				KTermPhrase kt = new KTermPhrase(ti.term);
				this.hashset.remove(kt);
				this.min = this.sortedList.peek().score;
			}

			//			// Search for the non-existent item
			//			int index = Collections.binarySearch(sortedList, v);      // -4
			//
			//			// Add the non-existent item to the list
			//			if (index < 0) {
			//				sortedList.add(-index-1, v);
			//			}else{
			//				sortedList.add(index, v);
			//			}
			ret = true;
		}
		return ret;
	}

	public boolean containsKey(KTermPhrase k){
		return this.hashset.contains(k);
	}

	public ArrayList<TermInfoNew> getArrayListWithScrambleQueries(){
		ArrayList<TermInfoNew> ret = new ArrayList<TermInfoNew>(this.sortedList.size());
		TermInfoNew ti;
		while((ti = this.sortedList.poll()) != null){
			ret.add(0, ti);
		}
		return ret;
	}

	public static void main(String[] args) throws Exception {
		Locale.setDefault(Locale.US);
		String bagWithScrambleTerms = "output1to3wordswin16eval10/new_life_baptist_church.www-1-5000-seed1";
		ResultOfAParticularTermNew r = new ResultOfAParticularTermNew();

		try {

			BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(bagWithScrambleTerms), "UTF-8"));

			String inputLine;
			int count =0;
			while ((inputLine = in.readLine()) != null) {
				TermInfoNew curr = new TermInfoNew();
				if(curr.parseTermInfo(inputLine)){
					if(count == 0){
						r.Term = curr;
						count++;
					}else{
						r.RelevantTerms.add(curr);
						count++;
					}
					//if(ThresholdOfTerms > 0 && count-1 >= ThresholdOfTerms) break;
				}
			}
		}catch(Exception e){
			System.err.println("ERROR when reading bag with Scramble terms and DocIDs:" + e.toString());
		}

		System.out.println("--- Read File!!!");
		MySortedHashMap h = new MySortedHashMap(500000); //takes 9 min for 2110000 records
		//HashMap<KTermPhrase,TermInfoNew> h = new HashMap<KTermPhrase,TermInfoNew>(); //takes 8 min for 2110000 records
		Random ra = new Random();
		while(r.RelevantTerms.size() > 0){
			TermInfoNew ti = r.RelevantTerms.remove(ra.nextInt(r.RelevantTerms.size()));
			KTermPhrase kt = new KTermPhrase(ti.term);
			if(!h.containsKey(kt)){
				h.put(kt, ti);
			}
		}
		
		r.RelevantTerms = h.getArrayListWithScrambleQueries();
		//Iterator<TermInfoNew> it = h.values().iterator();
		//while(it.hasNext()){
		//	r.RelevantTerms.add(it.next());
		//}
		//Collections.sort(r.RelevantTerms);
		
		r.printAllToFile(bagWithScrambleTerms+".new", r.getNumberOfRelevantTerms());
		r.printAll(50);

		//		Scanner sc = new Scanner(System.in);
		//		String str;
		//		
		//		while(!(str = sc.nextLine()).equals("exit")){
		//			TermInfoNew ti = new TermInfoNew();
		//			ti.parseTermInfo(str);
		//			KTermPhrase kt = new KTermPhrase(ti.term);
		//			
		//			if(!h.containsKey(kt)){
		//				h.put(kt, ti);
		//			}
		//		}
		//		
		//		ArrayList<TermInfoNew> li = h.getArrayListWithScrambleQueries();
		//		System.out.println("-------------------------------------");
		//		for(int i=0;i<li.size();i++){
		//			System.out.println(li.get(i));
		//		}
		//		System.out.println("-------------------------------------");

	}

}
