/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package lemur;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Locale;
import java.util.logging.Level;
import java.util.logging.Logger;
import sqlite.dataSources.Category;
import lemur.dataStructures.CategoryScoreRank;
import lemur.dataStructures.DocumentOfIndex;
import lemur.dataStructures.ListOfDocumentsInIndex;
import lemurproject.indri.DocumentVector;
import lemurproject.indri.IndexEnvironment;
import lemurproject.indri.ParsedDocument;
import lemurproject.indri.QueryEnvironment;
import lemurproject.indri.ScoredExtentResult;
import org.supercsv.cellprocessor.Optional;
import org.supercsv.cellprocessor.ParseInt;
import org.supercsv.cellprocessor.constraint.NotNull;
import org.supercsv.cellprocessor.ift.CellProcessor;
import org.supercsv.io.CsvListReader;
import org.supercsv.io.ICsvListReader;
import org.supercsv.prefs.CsvPreference;
import sqlite.CategoryDataSource;

/**
 *
 * @author drosatosgr
 */
public class createIndexBasedOnCategories {

    public static void main(String[] args) throws Exception {
        Locale.setDefault(Locale.US);
        final PrintStream out = new PrintStream(System.out, true, "UTF-8");

        String indexClueWeb = "files/www-1-5000-seed1/";
        String categoriesCSV = "files/www.similarweb.com_Categories.csv";
        String trecfile = ListOfDocumentsInIndex.class.getSimpleName()+".trec";
        int threshold = 300;
        String field = "description";

        /* Print help */
        if (args.length == 0) {
            out.println("Usage: java [-Xmx1024m] [-Djava.library.path=.] -jar createIndexBasedOnCategories.jar --index=/path/to/index/ --categoriesCSV=/path/to/file [--threshold=300] [--trecfile=/path/to/trecfile] [--field=description]");
            out.println("Notes: --index:           location MUST include the trailing slash");
            out.println("       --categoriesCSV:   file with the categories in CSV Format, seperator: ';'");
            out.println("       --threshold:       number of retrieved results per category, default value "+threshold);
            out.println("       --trecfile:        path where it will be stored the results as trectext, default value '"+trecfile+"'");
            out.println("       --field:           the value [name or description] from the CSV file that will be used as query, default value '"+field+"'");
            System.exit(0);
        }

        /* Read parameters */
        for (int i = 0; i < args.length; i++) {
            if (args[i].startsWith("--index=")) {
                indexClueWeb = args[i].substring(args[i].indexOf("=") + 1);
            } else if (args[i].startsWith("--trecfile=")) {
                trecfile = args[i].substring(args[i].indexOf("=") + 1);
            } else if (args[i].startsWith("--field=")) {
                field = args[i].substring(args[i].indexOf("=") + 1);
            } else if (args[i].startsWith("--categoriesCSV=")) {
                categoriesCSV = args[i].substring(args[i].indexOf("=") + 1);
            } else if (args[i].startsWith("--threshold=")) {
                threshold = Integer.valueOf(args[i].substring(args[i].indexOf("=") + 1));
            } else {
                out.println("Unrecognised parameter '" + args[i] + "'!");
                System.exit(0);
            }
        }

        Hashtable<Integer, Category> categ = loadCategoriesFromCSV(categoriesCSV);
        ListOfDocumentsInIndex listofdoc = new ListOfDocumentsInIndex();

        QueryEnvironment qenv = new QueryEnvironment();
        qenv.addIndex(indexClueWeb);

        List<Integer> idCateg = Collections.list(categ.keys());
        Collections.sort(idCateg);

        for (int i = 0; i < idCateg.size(); i++) {
            Category cat = categ.get(idCateg.get(i));
            out.println("--- Running Category: " + cat);
            
            ScoredExtentResult[] scResult = null;
            if(field.equalsIgnoreCase("name")){
                out.println("-- Query: " + cat.name.replaceAll("[^a-z\\sA-Z0-9]", " ").trim().toLowerCase());
                scResult = qenv.runQuery(cat.name.replaceAll("[^a-z\\sA-Z0-9]", " ").trim().toLowerCase(), threshold); //want: document = idDocumentClueWebIndex, score = LM score, order = rank of results
            }else if(field.equalsIgnoreCase("description")){
                out.println("-- Query: " + cat.description);
                scResult = qenv.runQuery(cat.description, threshold);
            }else{
                out.println("Unrecognised field '" + field + "'!");
                System.exit(0);
            }
            int[] ids = new int[scResult.length];
            
            int k=0;
            for(ScoredExtentResult ser: scResult){
                ids[k] = ser.document;
                k++;
            }
            
            String[] docno = qenv.documentMetadata(ids, "docno"); //want: docno = ClueWeb09_B ID
            DocumentVector[] dv = qenv.documentVectors(ids);
            
            //out.println("-- Results: " + scResult.length);
            
            Double tmpScoreOfFirstInRank = null;
            int count = 0;
            if(scResult.length > 0) tmpScoreOfFirstInRank = scResult[0].score;
            for (int r = 0; r < scResult.length; r++) {
                if (listofdoc.docs.containsKey(scResult[r].document)) {
                    listofdoc.docs.get(scResult[r].document).addCategory(new CategoryScoreRank(cat, scResult[r].score, tmpScoreOfFirstInRank, r + 1, scResult.length));
                } else {
                    DocumentOfIndex doi = new DocumentOfIndex(scResult[r].document, docno[r], documentVectorToString(dv[r]));
                    doi.addCategory(new CategoryScoreRank(cat, scResult[r].score, tmpScoreOfFirstInRank, r + 1, scResult.length));
                    listofdoc.docs.put(scResult[r].document, doi);
                    count++;
                }
            }
            out.println("-- Results: " + count +" / "+ scResult.length +" (Unique/Total Documents)");
        }

        out.println("- Total Documets: " + listofdoc.docs.size());
        listofdoc.saveObject();
        out.println("Saved Object '"+ListOfDocumentsInIndex.class.getSimpleName()+"' as "+ListOfDocumentsInIndex.class.getSimpleName()+".data");
        qenv.close();

        listofdoc.saveToTrecFormat(trecfile);
        out.println("Saved as trectext to file'"+trecfile+"'");
    }
    
    public static String documentVectorToString(DocumentVector v){
        String ret = "";
        for(int p: v.positions){
            ret += v.stems[p]+" ";
        }
        return ret;
    }

    public static Hashtable<Integer, Category> loadCategoriesFromCSV(String file) {
        Hashtable<Integer, Category> categ = new Hashtable<Integer, Category>();
        try {
            //CsvPreference SPACE_DELIMITED = new CsvPreference.Builder('"', ' ', "\r\n").build();
            CsvPreference STANDARD_PREFERENCE = new CsvPreference.Builder('"', ';', "\r\n").build();
            ICsvListReader inFile = new CsvListReader(new FileReader(file), STANDARD_PREFERENCE /*CsvPreference.STANDARD_PREFERENCE*/);
            final CellProcessor[] processors = new CellProcessor[]{
                new NotNull(new ParseInt()), //_id
                new NotNull(), // name
                new Optional(), // description
                new Optional(new ParseInt()), //parent_id
                new Optional(new ParseInt()), //forUploading
            };

            String[] header = inFile.getHeader(true);
            List<Object> customerList;
            while ((customerList = inFile.read(processors)) != null) {

                Integer _id = (Integer) customerList.get(0);
                String name = (String) customerList.get(1);
                String desc = (String) customerList.get(2);
                Integer parent_id = (Integer) customerList.get(3);
                Integer forUploading = (Integer) customerList.get(4);

                if (parent_id != null) {
                    categ.put(_id, new Category(_id, name, desc, parent_id, forUploading));
                }
            }

            inFile.close();

        } catch (FileNotFoundException ex) {
            Logger.getLogger(CategoryDataSource.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(CategoryDataSource.class.getName()).log(Level.SEVERE, null, ex);
        }
        return categ;
    }

}
