package scheduler;

import com.google.gson.JsonArray;
import com.google.gson.JsonParser;
import finals.Finals;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Hashtable;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;
import lemur.dataStructures.CategoryScoreRank;
import lemurproject.indri.QueryEnvironment;
import lemurproject.indri.ScoredExtentResult;
import sqlite.CategorizationDataSource;
import sqlite.MySQLiteHelper;
import sqlite.SearchDataSource;
import sqlite.dataSources.Categorization;
import sqlite.dataSources.Search;

public class CategorizerExecutionRunnable implements Runnable {

    SearchDataSource sds;
    CategorizationDataSource cds;
    QueryEnvironment qenv;

    public CategorizerExecutionRunnable(MySQLiteHelper db, String index) {
        this.sds = new SearchDataSource(db);
        this.cds = new CategorizationDataSource(db);
        this.qenv = new QueryEnvironment();
        try {
            this.qenv.addIndex(index);
        } catch (Exception ex) {
            Logger.getLogger(CategorizerExecutionRunnable.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void run() {
        try {
            if (this.qenv.documentCount() > 0) {
                ArrayList<Search> sea = sds.getUnCategorizedSearches();
                for (Search s : sea) {
                    String qqq = s.query.replaceAll("[^a-z\\sA-Z0-9]", " ").trim().toLowerCase();
                    ScoredExtentResult[] scResult = new ScoredExtentResult[0];
                    if(qqq.length() > 2){
                        scResult = qenv.runQuery(qqq, Finals.retrievalCutOff);
                    }
                    String[] doccateg = qenv.documentMetadata(scResult, "docat");

                    ArrayList<Categorization> results = new ArrayList<Categorization>();
                    if(Finals.kNNscoring == Finals.Method.AVGLMScore){
                        results = this.applyClassificationWithSimpleAverageOfScores(s, scResult, doccateg);
                    }else if(Finals.kNNscoring == Finals.Method.ExponentLMScore){
                        results = this.applyClassificationWithExponentOfScores(s, scResult, doccateg);
                    }
                    if (results.size() > 0) {
                        for (Categorization c : results) {
                            if (cds.insertNewCategorization(c)) {
                                s.categorized = 1;
                            } else {
                                s.categorized = 0;
                                break;
                            }
                        }
                    } else {
                        s.categorized = -1;
                    }
                     sds.setSearchAsCategorized(s);
                }
            }
        } catch (Exception ex) {
            Logger.getLogger(CategorizerExecutionRunnable.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public ArrayList<Categorization> applyClassificationWithSimpleAverageOfScores(Search s, ScoredExtentResult[] scResult, String[] doccateg) {
        ArrayList<Categorization> cat = new ArrayList<Categorization>();

        Hashtable<Integer, Double> countLeafs = new Hashtable<Integer, Double>();
        //Hashtable<Integer, Double> countRoots = new Hashtable<Integer, Double>();
        Double sumScore = 0.0;
        for (int r = 0; r < scResult.length; r++) {
            JsonArray ja = (JsonArray) new JsonParser().parse(doccateg[r]);
            for (int j = 0; j < ja.size(); j++) {
                CategoryScoreRank csr = new CategoryScoreRank(ja.get(j).getAsJsonObject());
                if (!countLeafs.containsKey(csr._id)) {
                    countLeafs.put(csr._id, scResult[r].score);
                } else {
                    countLeafs.replace(csr._id, countLeafs.get(csr._id) + scResult[r].score);
                }
                //if (!countRoots.containsKey(csr.parent_id)) {
                //    countRoots.put(csr.parent_id, scResult[r].score);
                //} else {
                //    countRoots.replace(csr.parent_id, countRoots.get(csr.parent_id) + scResult[r].score);
                //}
                sumScore += scResult[r].score;
            }
        }

        Comparator comp = new Comparator<Map.Entry<Integer, Double>>() {
            public int compare(Map.Entry<Integer, Double> o1, Map.Entry<Integer, Double> o2) {
                return o1.getValue().compareTo(o2.getValue());
            }
        };

        ArrayList<Map.Entry<Integer, Double>> leafs = new ArrayList(countLeafs.entrySet());
        Collections.sort(leafs, comp);

        //ArrayList<Map.Entry<Integer, Double>> roots = new ArrayList(countRoots.entrySet());
        //Collections.sort(roots, comp);

        //if (roots.size() > 0) {
        //    cat.add(new Categorization(null, s._id, roots.get(0).getKey().intValue(), 100.0 * roots.get(0).getValue().doubleValue() / sumScore, null));
        //}
        
        for(int i = 0; i < leafs.size() && i < Finals.topNCategories; i++){
            cat.add(new Categorization(null, s._id, leafs.get(i).getKey().intValue(), 100.0 * leafs.get(i).getValue().doubleValue() / sumScore, null));
        }
//        if (leafs.size() > 0) {
//            cat.add(new Categorization(null, s._id, leafs.get(0).getKey().intValue(), 100.0 * leafs.get(0).getValue().doubleValue() / sumScore, null));
//        }
//        if (leafs.size() > 1) {
//            cat.add(new Categorization(null, s._id, leafs.get(1).getKey().intValue(), 100.0 * leafs.get(1).getValue().doubleValue() / sumScore, null));
//        }

        return cat;
    }
    
        public ArrayList<Categorization> applyClassificationWithExponentOfScores(Search s, ScoredExtentResult[] scResult, String[] doccateg) {
        ArrayList<Categorization> cat = new ArrayList<Categorization>();

        Hashtable<Integer, Double> countLeafs = new Hashtable<Integer, Double>();
        //Double sumScore = 0.0;
        for (int r = 0; r < scResult.length; r++) {
            JsonArray ja = (JsonArray) new JsonParser().parse(doccateg[r]);
            for (int j = 0; j < ja.size(); j++) {
                CategoryScoreRank csr = new CategoryScoreRank(ja.get(j).getAsJsonObject());
                if (!countLeafs.containsKey(csr._id)) {
                    countLeafs.put(csr._id, Math.pow(Math.E, scResult[r].score));
                } else {
                    countLeafs.replace(csr._id, countLeafs.get(csr._id) + Math.pow(Math.E, scResult[r].score));
                }
            }
            //sumScore += Math.pow(Math.E, scResult[r].score);
        }

        Comparator comp = new Comparator<Map.Entry<Integer, Double>>() {
            public int compare(Map.Entry<Integer, Double> o1, Map.Entry<Integer, Double> o2) {
                return o1.getValue().compareTo(o2.getValue());
            }
        };

        ArrayList<Map.Entry<Integer, Double>> leafs = new ArrayList(countLeafs.entrySet());
        Collections.sort(leafs, comp);
 
        for(int i = 0; i < leafs.size() && i < Finals.topNCategories; i++){
            cat.add(new Categorization(null, s._id, leafs.get(leafs.size()-(i+1)).getKey().intValue(), leafs.get(leafs.size()-(i+1)).getValue().doubleValue(), null));
        }
//        if (leafs.size() > 0) {
//            cat.add(new Categorization(null, s._id, leafs.get(leafs.size()-1).getKey().intValue(), leafs.get(leafs.size()-1).getValue().doubleValue(), null));
//        }
//        if (leafs.size() > 1) {
//            cat.add(new Categorization(null, s._id, leafs.get(leafs.size()-2).getKey().intValue(), leafs.get(leafs.size()-2).getValue().doubleValue(), null));
//        }

        return cat;
    }

}
