package scheduler;

import com.google.gson.JsonArray;
import com.google.gson.JsonParser;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QueryParseException;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.update.UpdateExecutionFactory;
import com.hp.hpl.jena.update.UpdateFactory;
import com.hp.hpl.jena.update.UpdateProcessor;
import com.hp.hpl.jena.update.UpdateRequest;
import finals.Finals;
import java.awt.TrayIcon;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;
import lemur.dataStructures.CategoryScoreRank;
import lemurproject.indri.QueryEnvironment;
import lemurproject.indri.ScoredExtentResult;
import org.apache.jena.atlas.web.HttpException;
import org.apache.jena.atlas.web.auth.HttpAuthenticator;
import org.apache.jena.atlas.web.auth.SimpleAuthenticator;
import sqlite.CategorizationDataSource;
import sqlite.CategoryDataSource;
import sqlite.MySQLiteHelper;
import sqlite.SearchDataSource;
import sqlite.dataSources.Categorization;
import sqlite.dataSources.Search;
import virtuoso.AES;

public class RDFSenderExecutionRunnable implements Runnable {

    SearchDataSource sds;
    CategoryDataSource catds;
    CategorizationDataSource cds;

    public RDFSenderExecutionRunnable(MySQLiteHelper db) {
        this.sds = new SearchDataSource(db);
        this.catds = new CategoryDataSource(db);
        this.cds = new CategorizationDataSource(db);
    }

    public void run() {
        try {
            ArrayList<Categorization> cea = cds.getNonUploadedCategorizations();
            HashSet<Integer> cats = catds.getCategoriesIDsForUploading();
            HttpAuthenticator authenticator = null;
            String username = "";
            for (Categorization c : cea) {

                if (cats.contains(c.category_id)) {
                    if (authenticator == null) {
                        username = AES.symmetricDecrypt(Finals.userCARRE, Finals.seed);
                        if(username.equals("")) break;
                        authenticator = new SimpleAuthenticator(username, AES.symmetricDecrypt(Finals.passCARRE, Finals.seed).toCharArray());
                    }

                    int code = uploadToPrivateRDFandReturnStatusCode(c, authenticator, username);
                    if (code == 200) {
                        c.uploaded = 1;
                        cds.setCategorizationAsUploaded(c);
                    } else {
                        String mesage = "General Unknown Error!\nStatus Code: " + code;
                        if (code == 401) {
                            mesage = "Error in user authentication!\nPlease check your account.";
                        } else if (code == 404) {
                            mesage = "File not found!\nError on URL of Endpoint.";
                        } else if (code == 500) {
                            mesage = "SPARQL request error!";
                        } else if (code == -1) {
                            mesage = "General networking error!";
                        } else if (code == -2) {
                            mesage = "General error!";
                        }
                        Finals.trayIcon.displayMessage("Error in Uploading Process to RDF", mesage, TrayIcon.MessageType.ERROR);                    
                        Logger.getLogger(RDFSenderExecutionRunnable.class.getName()).log(Level.SEVERE, "Error in Uploading Process to RDF", mesage);
                        break;
                        //401 Unauthorized -> Error on username or password
                        //404 File not found -> Error on url endpoint
                        //500 SPARQL Request error
                        //-1 java.net.UnknownHostException -> network error    
                    }
                } else {
                    c.uploaded = -1;
                    cds.setCategorizationAsUploaded(c);
                }
            }
        } catch (Exception ex) {
            Logger.getLogger(RDFSenderExecutionRunnable.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public int uploadToPrivateRDFandReturnStatusCode(Categorization c, HttpAuthenticator authenticator, String username) {

        Query query = QueryFactory.create("SELECT ?s FROM <"+Finals.urlPrivateIRI + username+"> WHERE { ?s <"+Finals.urlOWL+"#has_connection> <"+Finals.urlPrivateIRI + username+"/connections/"+Finals.appUUID+"> }");
        QueryExecution qexec = QueryExecutionFactory.sparqlService(Finals.urlPrivateRDF, query, authenticator);
        ResultSet results = null;
        try{
            results = qexec.execSelect();
        } catch (HttpException he) {
            return he.getResponseCode();
            //401 Unauthorized -> Error on username or password
            //404 File not found -> Error on url endpoint
            //500 SPARQL Request error
            //-1 java.net.UnknownHostException -> network error            
        } catch (Exception ee) {
            Logger.getLogger(RDFSenderExecutionRunnable.class.getName()).log(Level.SEVERE, null, ee);
            return -2;
        }
        if (results != null && !results.hasNext()) {
                try {
            UpdateRequest request1 = UpdateFactory.create("INSERT DATA { GRAPH <" + Finals.urlPrivateIRI + username + "> { "
                    + "<"+Finals.urlPrivateIRI + username+">  <"+Finals.urlOWL+"#has_connection>  <"+Finals.urlPrivateIRI + username+"/connections/"+Finals.appUUID+"> . "
                    + "<"+Finals.urlPrivateIRI + username+"/connections/"+Finals.appUUID+">  <"+Finals.urlOWL+"#has_provider>  <http://carre.kmi.open.ac.uk/providers/web_queries> "
                    + "} }");
            UpdateProcessor proc = UpdateExecutionFactory.createRemoteForm(request1, Finals.urlPrivateRDF, authenticator);
            try {
                proc.execute();
            } catch (HttpException he) {
                return he.getResponseCode();
                //401 Unauthorized -> Error on username or password
                //404 File not found -> Error on url endpoint
                //500 SPARQL Request error
                //-1 java.net.UnknownHostException -> network error            
            } catch (Exception ee) {
                Logger.getLogger(RDFSenderExecutionRunnable.class.getName()).log(Level.SEVERE, null, ee);
                return -2;
            }

        } catch (QueryParseException qpe) {
            Logger.getLogger(RDFSenderExecutionRunnable.class.getName()).log(Level.SEVERE, null, qpe);
            return -2;
        }            
        }
        qexec.close();
        
        ArrayList<Object> info = cds.getInfoForRDF(c);

        try {
            String uuid = UUID.randomUUID().toString();
            
            String insert = "INSERT DATA { GRAPH <" + Finals.urlPrivateIRI + username + "> { "
                    + "\n<"+Finals.urlPrivateIRI + username+"/measurements/"+uuid+">  <"+Finals.urlOWL+"#has_date>  <"+Finals.urlPrivateIRI + username+"/measurements/"+uuid+"_date> . "
                    + "\n<"+Finals.urlPrivateIRI + username+"/measurements/"+uuid+"_date>  <"+Finals.urlOWL+"#has_value>  \""+(new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'")).format(new Date(((Long)info.get(1))))+"\"^^<http://www.w3.org/2001/XMLSchema#datetime> . "
                    + "\n<"+Finals.urlPrivateIRI + username+"/measurements/"+uuid+">  <"+Finals.urlOWL+"#has_weight>  <"+Finals.urlPrivateIRI + username+"/measurements/"+uuid+"_weight> . "
                    + "\n<"+Finals.urlPrivateIRI + username+"/measurements/"+uuid+"_weight>  <"+Finals.urlOWL+"#has_value>  \""+(Integer)info.get(2)+"\"^^<http://www.w3.org/2001/XMLSchema#integer> . "
                    + "\n<"+Finals.urlPrivateIRI + username+"/measurements/"+uuid+">  <"+Finals.urlOWL+"#is_provided_by>  <http://carre.kmi.open.ac.uk/providers/web_queries> . "
                    + "\n<"+Finals.urlPrivateIRI + username+"/measurements/"+uuid+">  <"+Finals.urlOWL+"#has_connection>  <"+Finals.urlPrivateIRI + username+"/connections/"+Finals.appUUID+"> . "
                    + "\n<"+Finals.urlPrivateIRI + username+"/measurements/"+uuid+">  <"+Finals.urlOWL+"#has_intention>  <"+Finals.urlOWL+"#"+((String)info.get(0)).replace(" ", "_").toLowerCase()+"> "
                    + "} }";
                    
            UpdateRequest request1 = UpdateFactory.create(insert);
            UpdateProcessor proc = UpdateExecutionFactory.createRemoteForm(request1, Finals.urlPrivateRDF, authenticator);
            try {
                proc.execute();
                return 200;
            } catch (HttpException he) {
                return he.getResponseCode();
                //401 Unauthorized -> Error on username or password
                //404 File not found -> Error on url endpoint
                //500 SPARQL Request error
                //-1 java.net.UnknownHostException -> network error            
            } catch (Exception ee) {
                Logger.getLogger(RDFSenderExecutionRunnable.class.getName()).log(Level.SEVERE, null, ee);
                return -2;
            }

        } catch (QueryParseException qpe) {
            Logger.getLogger(RDFSenderExecutionRunnable.class.getName()).log(Level.SEVERE, null, qpe);
            return -2;
        }
    }

}
