/*
 *  $Id: VirtuosoSPARQLExample1.java,v 1.4 2008/04/10 07:26:30 source Exp $
 *
 *  This file is part of the OpenLink Software Virtuoso Open-Source (VOS)
 *  project.
 *
 *  Copyright (C) 1998-2008 OpenLink Software
 *
 *  This project is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; only version 2 of the License, dated June 1991.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 */

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QueryParseException;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.update.UpdateExecutionFactory;
import com.hp.hpl.jena.update.UpdateFactory;
import com.hp.hpl.jena.update.UpdateProcessor;
import com.hp.hpl.jena.update.UpdateRequest;
import org.apache.jena.atlas.web.HttpException;
import org.apache.jena.atlas.web.auth.HttpAuthenticator;
import org.apache.jena.atlas.web.auth.SimpleAuthenticator;

public class VirtuosoSPARQLExample1 {

    /**
     * Executes a SPARQL query against a virtuoso url and prints results.
     */
    public static void main(String[] args) {

        HttpAuthenticator authenticator = new SimpleAuthenticator("drosatosgr", "g4680457d".toCharArray());

        try {
            UpdateRequest request1 = null; //UpdateFactory.create("INSERT DATA { GRAPH <https://carre.kmi.open.ac.uk/users/drosatosgr1> { <http://example/book5>  <http://purl.org/dc/elements/1.1/title>  \"Tara tatam 13\" } }");
            UpdateProcessor proc = UpdateExecutionFactory.createRemoteForm(request1, "http://carre.kmi.open.ac.uk:8890/sparql-auth", authenticator);
            try {
                proc.execute();
            } catch (HttpException he) {
                System.out.println(he.getResponseCode());
                System.out.println(he.getMessage());
                //401 Unauthorized -> Error on username or password
                //404 File not found -> Error on url endpoint
                //500 SPARQL Request error
                //-1 java.net.UnknownHostException -> network error            
            } catch (Exception ee) {
                System.out.println(ee.getMessage());
                //General Error
            }

        } catch (QueryParseException qpe) {
            System.out.println(qpe.getMessage());
            //Error on parse query
        }

        UpdateRequest request2 = UpdateFactory.create("DELETE DATA { GRAPH <https://carre.kmi.open.ac.uk/users/drosatosgr> { <http://example/book4>  <http://purl.org/dc/elements/1.1/title>  \"Tara tatam 12\" } }");
        UpdateProcessor proc2 = UpdateExecutionFactory.createRemoteForm(request2, "http://carre.kmi.open.ac.uk:8890/sparql-auth", authenticator);
        proc2.execute();

        String sparqlQueryString1 = "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>\n"
                + "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n"
                + "PREFIX carreSensors: <http://carre.kmi.open.ac.uk/ontology/sensors.owl>\n"
                + "PREFIX carreManufacturers: <http://carre.kmi.open.ac.uk/manufacturers>\n"
                + "PREFIX carreUsers: <https://carre.kmi.open.ac.uk/users/>\n"
                + "\n"
                + "SELECT ?subject ?predicate ?object FROM <https://carre.kmi.open.ac.uk/users/drosatosgr> WHERE { ?subject ?predicate ?object }";

        Query query = QueryFactory.create(sparqlQueryString1);
        QueryExecution qexec = QueryExecutionFactory.sparqlService("http://carre.kmi.open.ac.uk:8890/sparql-auth", query, authenticator);

        ResultSet results = qexec.execSelect();
        ResultSetFormatter.out(System.out, results, query);

        qexec.close();

    }
}
